/*--------------------------------------------------------------------------
 *  Copyright 2008 Taro L. Saito
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *--------------------------------------------------------------------------*/
//--------------------------------------
// XerialJ
//
// WikiToHTMLWriter.java
// Since: Apr 22, 2008 1:36:08 PM
//
// $URL: https://www.xerial.org/svn/project/XerialJ/trunk/xerial-wiki/src/main/java/org/xerial/wiki/WikiToHTMLWriter.java $
// $Author: leo $
//--------------------------------------
package org.xerial.wiki;

import java.io.IOException;
import java.io.StringWriter;
import java.io.Writer;
import java.util.Properties;

import org.xerial.util.log.Logger;
import org.xerial.util.template.Template;
import org.xerial.wiki.html.HTMLAttribute;

/**
 * Wiki to HTML coverter
 * 
 * @author leo
 * 
 */
public class WikiToHTMLWriter extends WikiWriterBase
{
    private static Logger _logger = Logger.getLogger(WikiToHTMLWriter.class);

    private final StringWriter htmlBuffer = new StringWriter();
    private final HTMLWriter writer;
    private boolean isInSection = false;
    private Properties properties = new Properties();

    public WikiToHTMLWriter() throws WikiException
    {
        this.writer = new HTMLWriter(htmlBuffer);
        init();
    }

    public void render(Writer writer) throws WikiException
    {
        try
        {
            Template template = new Template(this.getClass().getResourceAsStream("/org/xerial/wiki/WikiHTML.template"));
            properties.setProperty("wikiContent", htmlBuffer.toString());
            String html = template.apply(properties);
            writer.append(html);
        }
        catch (IOException e)
        {
            throw new WikiException(WikiErrorCode.IOError, e.getMessage());
        }
    }

    protected void init() throws WikiException
    {
        setTitle("wiki");
    }

    protected void startTag(String tagName) throws WikiException
    {
        try
        {
            writer.startTag(tagName);
        }
        catch (IOException e)
        {
            throw new WikiException(WikiErrorCode.IOError, e.getMessage());
        }
    }

    protected void startTag(String tagName, HTMLAttribute attribute) throws WikiException
    {
        try
        {
            writer.startTag(tagName, attribute);
        }
        catch (IOException e)
        {
            throw new WikiException(WikiErrorCode.IOError, e.getMessage());
        }
    }

    protected void endTag(String tagName) throws WikiException
    {
        try
        {
            writer.endTag(tagName);
        }
        catch (IOException e)
        {
            throw new WikiException(WikiErrorCode.IOError, e.getMessage());
        }
    }

    protected void writeText(String text) throws WikiException
    {
        try
        {
            writer.text(text);
        }
        catch (IOException e)
        {
            throw new WikiException(WikiErrorCode.IOError, e.getMessage());
        }
    }

    protected void selfCloseTag(String tagName) throws WikiException
    {
        try
        {
            writer.selfCloseTag(tagName);
        }
        catch (IOException e)
        {
            throw new WikiException(WikiErrorCode.IOError, e.getMessage());
        }
    }

    protected void enter() throws WikiException
    {
        try
        {
            writer.text("\n");
        }
        catch (IOException e)
        {
            throw new WikiException(WikiErrorCode.IOError, e.getMessage());
        }
    }

    public void beginBody() throws WikiException
    {
        startTag("body");
    }

    public void beginBold() throws WikiException
    {
        startTag("strong");
    }

    public void beginItaric() throws WikiException
    {
        startTag("i");
    }

    public void beginItem(int level) throws WikiException
    {
        startTag("li");
    }

    public void beginLink(String ref, String target) throws WikiException
    {
        HTMLAttribute attrib = new HTMLAttribute();
        attrib.add("href", ref);
        if (target != null)
            attrib.add("target", target);
        startTag("a", attrib);
    }

    public void beginSection(int level) throws WikiException
    {
        if (isInSection)
            endSection();
        startTag("section");
        isInSection = true;
    }

    public void setTitle(String title) throws WikiException
    {
        properties.setProperty("title", title);
    }

    public void beginVerbatim() throws WikiException
    {
        startTag("pre");
    }

    public void endBody() throws WikiException
    {
        endTag("body");
    }

    public void beginImage(String href, String label) throws WikiException
    {
        startTag("img", new HTMLAttribute().add("src", href).add("title", label));
    }

    public void endBold() throws WikiException
    {
        endTag("strong");
    }

    public void endItaric() throws WikiException
    {
        endTag("i");
    }

    public void endItem() throws WikiException
    {
        endTag("li");
        enter();
    }

    public void endLink() throws WikiException
    {
        endTag("a");
    }

    public void endSection() throws WikiException
    {
        if (isInSection)
            endTag("section");
        enter();
    }

    public void endVerbatim() throws WikiException
    {
        endTag("pre");
        enter();
    }

    public void text(String text) throws WikiException
    {
        writeText(text);
    }

    public void newLine() throws WikiException
    {
        // close items
        selfCloseTag("br");
        enter();
    }

    public void beginParagraph() throws WikiException
    {
        enter();
        startTag("p");
    }

    public void endParagraph() throws WikiException
    {
        endTag("p");
    }

    public void beginSectionTitle(int level) throws WikiException
    {
        enter();
        startTag("h" + level);
    }

    public void endSectionTitle(int level) throws WikiException
    {
        endTag("h" + level);
        enter();
    }

    public void endImage() throws WikiException
    {
        endTag("img");
        enter();
    }

    public void beginOrderedList() throws WikiException
    {
        startTag("ol");
    }

    public void beginUnorderedList() throws WikiException
    {
        startTag("ul");
    }

    public void endOrderedList() throws WikiException
    {
        endTag("ol");
    }

    public void endUnorderedList() throws WikiException
    {
        endTag("ul");
    }

    public void beginAnchor(String name) throws WikiException
    {
        startTag("a", new HTMLAttribute().add("name", name));
    }

    public void endAnchor() throws WikiException
    {
        endTag("a");
    }

    public void rawText(String text) throws WikiException
    {
        try
        {
            writer.rawText(text);
        }
        catch (IOException e)
        {
            throw new WikiException(WikiErrorCode.IOError, e.getMessage());
        }

    }
}