/*--------------------------------------------------------------------------
 *  Copyright 2008 Taro L. Saito
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *--------------------------------------------------------------------------*/
//--------------------------------------
// XerialJ
//
// WikiWalker.java
// Since: Apr 22, 2008 4:09:39 PM
//
// $URL: https://www.xerial.org/svn/project/XerialJ/trunk/xerial-wiki/src/main/java/org/xerial/wiki/WikiWalker.java $
// $Author: leo $
//--------------------------------------
package org.xerial.wiki;

import java.util.HashSet;
import java.util.List;

import org.xerial.core.XerialException;
import org.xerial.util.bean.BeanUtil;
import org.xerial.util.log.Logger;

/**
 * WikiWalker traverses an input {@link Wiki} object
 * 
 * @author leo
 * 
 */
public class WikiWalker
{
    private final static WikiPluginLoader pluginLoader = new WikiPluginLoader();

    private final WikiWriter writer;
    private static Logger _logger = Logger.getLogger(WikiWalker.class);

    private final HashSet<WikiElement> skipElement = new HashSet<WikiElement>();

    private static class WikiContextImpl implements WikiContext
    {
        WikiElement parent;
        WikiElement current;
        boolean isInVerbatimBlock = false;

        public WikiContextImpl(WikiElement parent, WikiElement current)
        {
            transitContext(parent, current);
        }

        public WikiElement getParentWikiElement()
        {
            return parent;
        }

        public WikiElement getCurrentWikiElement()
        {
            return current;
        }

        public WikiContext transitContext(WikiElement parent, WikiElement current)
        {
            this.parent = parent;
            this.current = current;
            return this;
        }

        public boolean isInVerbatimBlock()
        {
            return isInVerbatimBlock;
        }

        public void setVerbatimBlock(boolean isInVerbatim)
        {
            this.isInVerbatimBlock = isInVerbatim;
        }
    }

    public WikiWalker(WikiWriter writer)
    {
        this.writer = writer;
    }

    public void addSkipElement(WikiElement elementToSkip)
    {
        skipElement.add(elementToSkip);
    }

    /**
     * Walks the wiki instance, then invokes conversion methods in the given
     * {@link WikiWriter}
     * 
     * @param wiki
     * @throws WikiException
     */
    public void walk(WikiElement elem) throws WikiException
    {
        walkInternal(new WikiContextImpl(null, elem));
    }

    private void walkInternal(WikiContext context) throws WikiException
    {
        WikiElement parent = context.getParentWikiElement();
        WikiElement elem = context.getCurrentWikiElement();

        if (skipElement.contains(elem))
            return;

        switch (elem.getType())
        {
        case Root:
        {
            Wiki wiki = Wiki.class.cast(elem);
            writer.beginBody();
            for (WikiElement e : wiki.getWikiElementList())
            {
                walkInternal(context.transitContext(wiki, e));
            }
            writer.endSection();
            writer.endBody();
            break;
        }
        case WikiBlock:
        {
            WikiBlock wikiBlock = WikiBlock.class.cast(elem);
            for (WikiElement e : wikiBlock.getWikiElementList())
            {
                walkInternal(context.transitContext(wikiBlock, e));
            }
            break;
        }
        case WikiLine:
        {
            WikiLine wikiLine = WikiLine.class.cast(elem);

            // skip the blank wiki line
            if (wikiLine.getWikiElementList().size() <= 0)
                break;

            // iterates over wiki line components
            writer.beginParagraph();
            for (WikiElement e : wikiLine.getWikiElementList())
            {
                walkInternal(context.transitContext(wikiLine, e));
            }
            writer.endParagraph();
            break;
        }
        case BlankLine:
            writer.resetIndent(0);
            //writer.newLine();
            break;
        case Function:
        {
            WikiFunction wikiFunction = WikiFunction.class.cast(elem);

            String functionName = wikiFunction.getName();
            WikiPlugin functionPlugin = getPlugin(functionName);
            if (functionPlugin == null)
            {
                _logger.warn("unknown function: " + functionName);
                return;
            }

            // evaluate variable assignments
            try
            {
                BeanUtil.populateBeanWithMap(functionPlugin, wikiFunction.getAssignment());
            }
            catch (XerialException e)
            {
                _logger.error("binding failure: " + functionName + "(" + wikiFunction.getAssignment().toString() + ")");
            }

            List<WikiElement> argumentList = wikiFunction.getArgumentList();
            String[] functionArgs = new String[argumentList.size()];
            for (int i = 0; i < argumentList.size(); i++)
            {
                functionArgs[i] = eval(argumentList.get(i));
            }

            // run the plugin
            functionPlugin.eval(functionArgs, context.transitContext(parent, wikiFunction), writer);

            break;
        }
        case Text:
        {
            WikiText wikiText = WikiText.class.cast(elem);
            if (context.isInVerbatimBlock())
                writer.rawText(wikiText.toString());
            else
                writer.text(wikiText.toString());
            break;
        }
        default:
            _logger.warn("unknown element type: " + elem.getType());
        }
    }

    public static String eval(WikiElement elem)
    {
        StringBuilder builder = new StringBuilder();
        evalInternal(elem, builder);
        return builder.toString();
    }

    private static void evalInternal(WikiElement elem, StringBuilder builder)
    {
        switch (elem.getType())
        {
        case BlankLine:
        {
            builder.append("\n");
            break;
        }
        case WikiBlock:
        {
            WikiBlock wikiBlock = WikiBlock.class.cast(elem);
            for (WikiElement e : wikiBlock.getWikiElementList())
            {
                evalInternal(e, builder);
            }
            break;
        }
        case WikiLine:
        {
            // iterates over wiki line components
            WikiLine wikiLine = WikiLine.class.cast(elem);
            for (WikiElement e : wikiLine.getWikiElementList())
                evalInternal(e, builder);
            break;
        }
        case Function:
        {
            _logger.warn("nested function is not supported");
            break;
        }
        case Text:
        {
            WikiText wikiText = WikiText.class.cast(elem);
            builder.append(wikiText.toString());
            break;
        }
        default:
            _logger.warn("unknown element type: " + elem.getType());
        }

    }

    public WikiPlugin getPlugin(String pluginName)
    {
        return pluginLoader.getPlugin(pluginName);
    }

}
